// SPDX-License-Identifier: GPL-2.0+
/*
 * This empty file prevents make linking error.
 * No custom logic for dragonboard845c so far.
 *
 * (C) Copyright 2022 Sumit Garg <sumit.garg@linaro.org>
 */

void noop(void) {}
