/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Cadence DDR Driver
 *
 * Copyright (C) 2012-2021 Cadence Design Systems, Inc.
 * Copyright (C) 2018-2021 Texas Instruments Incorporated - https://www.ti.com/
 */

#ifndef LPDDR4_16BIT_OBJ_IF_H
#define LPDDR4_16BIT_OBJ_IF_H

#include "lpddr4_16bit_if.h"

#endif  /* LPDDR4_16BIT_OBJ_IF_H */
