/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2022 Broadcom Ltd.
 */

#ifndef __BCM963138_H
#define __BCM963138_H

#define CONFIG_SYS_SDRAM_BASE		0x00000000
#define CONFIG_SYS_HZ_CLOCK		500000000

#endif
