/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * GXP board
 *
 * (C) Copyright 2022 Hewlett Packard Enterprise Development LP.
 * Author: Nick Hawkins <nick.hawkins@hpe.com>
 * Author: Jean-Marie Verdun <verdun@hpe.com>
 */

#ifndef _GXP_H_
#define _GXP_H_

#define CONFIG_SYS_SDRAM_BASE   0x40000000

#endif
