/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2022 Kontron Electronics GmbH
 */

#ifndef __SL_MX6UL_COMMON_H
#define __SL_MX6UL_COMMON_H

bool sl_mx6ul_is_spi_nor_boot(void);

#endif // __SL_MX6UL_COMMON_H
