/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2010
 * Texas Instruments Incorporated.
 * Steve Sakoman  <steve@sakoman.com>
 *
 * Configuration settings for the TI OMAP4 Panda board.
 * See ti_omap4_common.h for OMAP4 common part
 */

#ifndef __CONFIG_PANDA_H
#define __CONFIG_PANDA_H

#include <configs/ti_omap4_common.h>

#endif /* __CONFIG_PANDA_H */
