/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Configuration for Xilinx Versal NET MINI configuration
 *
 * Copyright (C) 2018 - 2022, Xilinx, Inc.
 * Copyright (C) 2022, Advanced Micro Devices, Inc.
 *
 * Michal Simek <michal.simek@amd.com>
 */

#ifndef __CONFIG_VERSAL_NET_MINI_H
#define __CONFIG_VERSAL_NET_MINI_H

#define CONFIG_EXTRA_ENV_SETTINGS

#include <configs/xilinx_versal_net.h>

/* Undef unneeded configs */
#undef CONFIG_EXTRA_ENV_SETTINGS

#endif /* __CONFIG_VERSAL_NET_MINI_H */
