/* SPDX-License-Identifier: GPL-2.0 */
/*
 *  Copyright 2021 NXP
 */

#ifndef __DT_BINDINGS_IMX93_POWER_H__
#define __DT_BINDINGS_IMX93_POWER_H__

#define IMX93_POWER_DOMAIN_MLMIX		0
#define IMX93_POWER_DOMAIN_MEDIAMIX		1

#endif
