/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2020 Marek Vasut <marek.vasut@gmail.com>
 */

#ifndef __LPDDR4_TIMING_H__
#define __LPDDR4_TIMING_H__

extern struct dram_timing_info dram_timing_1gb;
extern struct dram_timing_info dram_timing_2gb;
extern struct dram_timing_info dram_timing_3gb;
extern struct dram_timing_info dram_timing_4gb;

#endif /* __LPDDR4_TIMING_H__ */
