/* SPDX-License-Identifier: MIT */
/*
 *  Copyright(c) 2015 - 2020 Xilinx, Inc.
 *
 *  Jorge Ramirez-Ortiz <jorge@foundries.io>
 */

#ifndef __ARCH_ZYNQMP_ECC_INIT_H
#define __ARCH_ZYNQMP_ECC_INIT_H

void zynqmp_ecc_init(void);

#endif
