/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2022 Broadcom Ltd.
 */

#ifndef __BCM947622_H
#define __BCM947622_H

#define CONFIG_SYS_SDRAM_BASE		0x00000000

#define COUNTER_FREQUENCY		50000000
#endif
